/*
    ChibiOS/RT - Copyright (C) 2006,2007,2008,2009,2010,
                 2011,2012 Giovanni Di Sirio.

    This file is part of ChibiOS/RT.

    ChibiOS/RT is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    ChibiOS/RT is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

                                      ---

    A special exception to the GPL can be applied should you wish to distribute
    a combined work that includes ChibiOS/RT, without being obliged to provide
    the source code for any proprietary components. See the file exception.txt
    for full details of how and when the exception can be applied.
*/

/**
 * @file    Cortex-A8/chtypes.h
 * @brief   Cortex-A8 architecture port system types.
 *
 * @addtogroup ARM_CORE
 * @{
 */

#include "chconf.h"

#define FALSE 0
#define TRUE 1

#if !defined(__DOXYGEN__)

.set    MODE_SYS, 0x1F

.text

/*
 * @brief   Performs a context switch between two threads.
 * @details This is the most critical code in any port, this function
 *          is responsible for the context switch between 2 threads.
 * @note    The implementation of this code affects <b>directly</b> the context
 *          switch performance so optimize here as much as you can.
 *
 * @param[in] ntp       the thread to be switched in
 * @param[in] otp       the thread to be switched out
 */
.balign 16
.code 32
.global _port_switch
_port_switch:       /* void port_switch(Thread *ntp, Thread *otp) */
        /* Store the old data */    
        stmfd   sp!, {r4, r5, r6, r7, r8, r9, r10, r11, lr}

#if CORTEX_USE_FPU        
        vpush {s16-s31}
        vmrs r4, fpscr
        push {r4} 
#endif /* CORTEX_USE_FPU */

        str     sp, [r1, #12]   /* <= offset to p_ctx */

        /*****************************************************/         
        
        /* Load the new data */
        ldr     sp, [r0, #12]   /* <= offset to p_ctx */

#if CORTEX_USE_FPU        
        pop {r4}
        vmsr fpscr, r4
        vpop {s16-s31}
#endif /* CORTEX_USE_FPU */
        
        ldmfd   sp!, {r4, r5, r6, r7, r8, r9, r10, r11, pc}
        
        
        
/*
 * @brief   Start a thread by invoking its work function.
 * @details If the work function returns @p chThdExit() is automatically
 *          invoked.
 * @note    The threads always start in ARM mode and then switches to the
 *          thread-function mode.
 */

.balign 16
.code 32
.globl _port_thread_start
_port_thread_start:
#if CH_DBG_SYSTEM_STATE_CHECK
        mov     r0, #0
        ldr     r1, =dbg_lock_cnt
        str     r0, [r1]
#endif
        msr     CPSR_c, #MODE_SYS
        mov     r0, r5
        mov     lr, pc
        bx      r4
        bl      chThdExit

#endif /* !defined(__DOXYGEN__) */

/** @} */
